(function() {
    function OCFSPI(packetParser, sendPacket) {
        OCFBaseEx.apply(this, arguments);
        this.handlers = [
            reply_SPI_Enable,
            reply_SPI_Config,
            reply_SPI_WriteAndRead,
            reply_SPI_CaptureSample
        ];
    };
    OCFSPI.prototype = Object.create(OCFBaseEx.prototype);
    OCFSPI.prototype.constructor = OCFSPI;
    window.OCFSPI = OCFSPI;

    OCFSPI.prototype.initSymbolsForDevice = function(settings, registerModel) {
        OCFBaseEx.prototype.initSymbolsForDevice.apply(this, arguments);
    };

    OCFSPI.prototype.init = function(info) {
        this.cs_gpio = info.cs_gpio;
        return OCFBaseEx.prototype.init.apply(this, arguments);
    };

    OCFSPI.prototype.read = function(regInfo) {
        var cs_gpio = regInfo.cs_gpio || this.cs_gpio;
        var num_bytes = parseInt(regInfo.num_bytes);
        var dummy = [];
        for (var idx=0; idx<num_bytes; idx++)  dummy.push(0);
        gc.console.debug('OCFSPI', 'read regInfo cs_gpio ' + cs_gpio + ' num_bytes ' + num_bytes);
        return this.SPI_WriteAndRead(this.unit, cs_gpio, dummy, num_bytes);
    };

    OCFSPI.prototype.write = function(regInfo, data) {
        var cs_gpio = regInfo.cs_gpio || this.cs_gpio;
        gc.console.debug('OCFSPI', 'write regInfo cs_gpio ' + cs_gpio + ' data ' + data);
        return this.SPI_WriteAndRead(this.unit, cs_gpio, data, data.length);
    };

    OCFSPI.prototype.ensureConfigured = function(config_seq) {
        var self = this;
        return Q.Promise(function(resolve, reject) {
            var promises = [];
            var seq_len = config_seq ? config_seq.length : 0;
            for (var i=0; i<seq_len; i++) {
                var config = config_seq[i];
                switch (config.command) {
                    case 'enable':
                        promises.push(self.SPI_Enable(config.unit || self.unit, config.enable));
                        break;

                    case 'config':
                        promises.push(self.SPI_Config(config.unit || self.unit, config.bitrate, config.protocol, config.datawidth, config.cs_mode, config.cs_change));
                        break;
                }
            }
            return Q.all(promises).then(resolve).fail(reject);
        }.bind(this));
    };
    
    // Reference ocf_common.h
    //#define SPI_Interface       0x05
    //typedef enum
    //{
    //    ocCmd_SPI_Enable = 0x00,
    //    ocCmd_SPI_Config,
    //    ocCmd_SPI_WriteAndRead,
    //    ocCmd_SPI_CaptureSample,
    //} SPI_CMD;
    var SPI_Interface = 0x05;
    var ocCmd_SPI_Enable = 0x00;
    var ocCmd_SPI_Config = 0x01;
    var ocCmd_SPI_WriteAndRead = 0x02;
    var ocCmd_SPI_CaptureSample = 0x03;
    var ocCmd_Sys_DevCtrl = 0xC;
    
    OCFSPI.prototype.SPI_Enable = function(unit, enable) {
        //uint32_t unit, bool enable
        var params = [this.uint32_leb4(unit), this.uint32_leb4(enable ? 1 : 0)];
        return this.h2c_command(SPI_Interface, unit, ocCmd_SPI_Enable, params);
    };
    var reply_SPI_Enable = function(self, qdef, unit, status, replypkt) {
        if (status == 0) {
            var enable = self.leb4_uint32(replypkt.params[1]) == 1;
            self.unit_state[unit].current = enable ? 1 : 0;
            qdef && qdef.resolve(enable);
        } else {
            qdef && qdef.reject(self.status_msg(status, replypkt.payload ? self.bytes_ascii(replypkt.payload) : 'Failed in Enable/Disable'));
        }
    };
    OCFSPI.prototype.SPI_Config = function(unit, bitrate, protocol, datawidth, cs_mode, cs_change) {
        //uint32_t unit, uint32_t bit_rate, uint32_t protocol (spi mode), uint32_t datawidth, uint32_t cs_mode, uint32_t cs_change
        // bitrate: in units of 1kHz; max rate is 60000 kHz
        // protocol: see ssi.h
        //  #define SSI_FRF_MOTO_MODE_0  0x00000000 // Moto fmt, polarity 0, phase 0
        //  #define SSI_FRF_MOTO_MODE_1  0x00000002 // Moto fmt, polarity 0, phase 1
        //  #define SSI_FRF_MOTO_MODE_2  0x00000001 // Moto fmt, polarity 1, phase 0
        //  #define SSI_FRF_MOTO_MODE_3  0x00000003 // Moto fmt, polarity 1, phase 1
        //  #define SSI_FRF_TI           0x00000010 // TI frame format
        //  #define SSI_FRF_NMW          0x00000020 // National microwire frame format
        //  #define SSI_ADV_MODE_BI_READ   0x00000140
        //  #define SSI_ADV_MODE_BI_WRITE  0x00000040
        // datawidth: word width between 4 and 16 bits
        // cs_mode: SPI active mode: 1 active high; 0 active low
        // cs_change: cs_mode change between SPI word. 0: no change; 1: change
        var params = [this.uint32_leb4(unit), this.uint32_leb4(bitrate), this.uint32_leb4(protocol),
                      this.uint32_leb4(datawidth), this.uint32_leb4(cs_mode), this.uint32_leb4(cs_change)];
        return this.h2c_command(SPI_Interface, unit, ocCmd_SPI_Config, params);
    };
    var reply_SPI_Config = function(self, qdef, unit, status, replypkt) {
        if (status == 0) {
            self.unit_state[unit].current = 2;
            qdef && qdef.resolve(true);
        } else {
            qdef && qdef.reject(self.status_msg(status, replypkt.payload ? self.bytes_ascii(replypkt.payload) : 'Failed in Config'));
        }
    };
    OCFSPI.prototype.SPI_WriteAndRead = function(unit, cs_gpio, pWriteBuffer, num_bytes) {
        //uint8_t unit, uint32_t cs_gpio, uint16_t num_bytes,
        //cs_gpio: the gpio pin to be used as the chip select. value is 1<<n, where n=[0..15]
        //pWriteBuffer is uint8_t*, contains data to write
        //num_bytes is number of bytes to write and to read. By SPI design, both write and read bounds have the same number of bytes.
        // firmware 0.3.0.3, cs_gpio is port pin name e.g. 'PK1'
        // firmware 0.9, may not have cs_gpio. Review code for any change.
        var gpio;
        if (typeof cs_gpio == 'string') {
            gpio = cs_gpio.split('').map(v => {return v.charCodeAt(0);});
        } else {
            gpio = this.uint32_leb4(cs_gpio); // in case it is a pin mask
        }
        var params = [this.uint32_leb4(unit), gpio, this.uint16_leb4(pWriteBuffer && pWriteBuffer.length || num_bytes)];
        return this.h2c_command(SPI_Interface, unit, ocCmd_SPI_WriteAndRead, params, pWriteBuffer);
    };
    var reply_SPI_WriteAndRead = function(self, qdef, unit, status, replypkt) {
        if (!qdef) return;
        if (status == 0) {
            qdef.resolve(replypkt.payload);
        } else {
            qdef.reject(self.status_msg(status, replypkt.payload ? self.bytes_ascii(replypkt.payload) : 'Failed in WriteAndRead'));
        }
    };
    OCFSPI.prototype.SPI_CaptureSample = function(unit, bytes_to_write, sample_size, payload, options) {
        //uint8_t unit, uint16_t bytes_to_write, unit32_t sample_size
        // bytes_to_write: each_frame_bytes, one_sample_output_bytes * channels
        // sample_size: number of samples to capture for a channel
        // payload is the output data for getting a sample for each channel
        // so, total_output_bytes after capture done = sample_size * each_frame_bytes
        //
        // The following is not part of DLL APIs
        // options is {
        //     callback: <is a callback function of signature function(replypkt) > for streaming approach; undefined for buffering approach
        // }
        // Streaming approach is to address potential performance problems on browser side. If total_output_bytes
        // is 5K+ bytes, or if captue sample request is made at high rate regardless of the size of total_output_bytes,
        // it is better to use streaming approach.
        // Since some applications can do processing without waiting the entire capture, or may suffer performance
        // issues from allocating a huge buffer to hold data, this option is provided to enable applications to
        // make design decisioins.
        // In streaming approach, callback is invoked for each data packet received from firmware.
        // In buffering approach, application will get all the captured data through Q promise
        // after capture done packect is recieved.
        var params = [this.uint32_leb4(unit), this.uint32_leb4(bytes_to_write), this.uint16_leb4(sample_size)];
        this.capture_sample_options = options || {};
        this.capture_payload = [];
        return this.h2c_command(SPI_Interface, unit, ocCmd_SPI_CaptureSample, params, payload);
    };
    var reply_SPI_CaptureSample = function(self, qdef, unit, status, replypkt, packet_type) {
        // SPI_interface.c, and ocf_common.h; status 1: capture in progress, 2: capture sample done
        // firmware version 0.3.0.3 first reply is status packet;
        // if capture is done without issues, firmware sends at least 1 data packet
        if (qdef) { self.capture_qdef = qdef; } // firmware 0.3.0.3 - unit for data packet is 0 for unknown reason
        if (status >= 1 && status <= 2) {
            // packet type: DATA_PACKET=0x0002, STATUS_PACKET=0x0003
            if (self.capture_sample_options.callback) {
                self.capture_sample_options.callback(replypkt);
            } else if (replypkt.payload) {
                if (packet_type == 0x2) {
                    if (self.leb2_uint16(replypkt.packet_num) == 0x1) self.capture_payload = [];
                    Array.prototype.push.apply(self.capture_payload, replypkt.payload);
                }
            }
            if (status == 2) {
                self.capture_qdef && self.capture_qdef.resolve(self.capture_payload);
                self.capture_qdef = self.capture_payload = undefined;
            }
        } else if (status != 0) {
            self.capture_qdef && self.capture_qdef.reject(self.status_msg(status, replypkt.payload ? self.bytes_ascii(replypkt.payload) : 'Failed in CaptureSample'));
            self.capture_qdef = self.capture_payload = undefined;
        }
    };
    OCFSPI.prototype.SPI_DevCtrl = function(unit, active_mode, input_trigger_gpio_active, input_trigger_gpio,
        ctrl_pin_ary, sample_rate, sample_bits, dma_arb_size) {
        // params is unit, sizeof dev_ctrl; payload length is sizeof dev_ctrl, payload is dev_ctrl
        // dev_ctrl is a struct of
        //    uint8 active_mode, uint8 input_trigger_gpio_active, unit16 input_trigger_gpio,
        //    CTRL_PIN ctrl_pin[3],
        //    unit32 sample_rate, unit8 sample_bits, uint8 dma_arb_size
        // each CTRL_PIN is a struct of
        //    unit16 gpio_pin, uint16 pin_type, uint16 control_level, int16 invert_clk
        var payload = [active_mode & 0xff, input_trigger_gpio_active & 0xff];
        Array.prototype.push.apply(payload, this.uint16_leb2(input_trigger_gpio));
        for (var idx=0; idx<3; idx++) {
            if (idx < ctrl_pin_ary.length) {
                var gpio_pin = undefined;
                if (typeof ctrl_pin_ary[idx].gpio_pin == 'string') { // firmware version 0.3.0.3 gpio pin is string, e.g. 'K1'
                    gpio_pin = ctrl_pin_ary[idx].gpio_pin.split('').map(v => {return v.charCodeAt(0);});
                }
                Array.prototype.push.apply(payload, gpio_pin || this.uint16_leb2(ctrl_pin_ary[idx].gpio_pin) );
                Array.prototype.push.apply(payload, this.uint16_leb2(ctrl_pin_ary[idx].pin_type) );
                Array.prototype.push.apply(payload, this.uint16_leb2(ctrl_pin_ary[idx].control_level) );
                Array.prototype.push.apply(payload, this.uint16_leb2(ctrl_pin_ary[idx].invert_clk) );
            } else {
                Array.prototype.push.apply(payload, this.uint16_leb2(0) );
                Array.prototype.push.apply(payload, this.uint16_leb2(3) );// INVALID_PIN
                Array.prototype.push.apply(payload, this.uint16_leb2(0) );
                Array.prototype.push.apply(payload, this.uint16_leb2(0) );
            }
        }
        Array.prototype.push.apply(payload, this.uint32_leb4(sample_rate) );
        payload.push( sample_bits & 0xff );
        payload.push( dma_arb_size & 0xff  );
        var params = [this.uint32_leb4(unit), this.uint32_leb4(payload.length)];
        return this.h2c_command(SPI_Interface, unit, ocCmd_Sys_DevCtrl, params, payload);
    };
    var reply_SPI_DevCtrl = function(self, qdef, unit, status, replypkt) {
        if (!qdef) return;
        if (status == 0) {
             qdef.resolve(replypkt.payload ? replypkt.payload : true);
        } else {
            qdef.reject(self.status_msg(status, replypkt.payload ? self.bytes_ascii(replypkt.payload) : 'Failed in DevCtrl'));
        }
    };

})();

